@@ -1,6 +1,5 @@
-import logging
+
 import random
-from copy import copy
 
 import numpy as np
 import tensorflow as tf
@@ -11,6 +10,8 @@ from .bert_utils import (create_instances_from_document,
 
 from transformers import PreTrainedTokenizer
 
+LOGGER = tf.get_logger()
+
 
 def seq_tag_label_handling(tokenized_dict, target, pad_token):
     special_token_mask = tokenized_dict['special_tokens_mask']
@@ -85,10 +86,10 @@ def _create_bert_features(problem,
                           problem_type,
                           is_seq):
 
-    for example in example_list:
-        try:
+    for example_id, example in enumerate(example_list):
+        if mode != tf.estimator.ModeKeys.PREDICT:
             raw_inputs, raw_target = example
-        except ValueError:
+        else:
             raw_inputs = example
             raw_target = None
 
@@ -118,7 +119,7 @@ def _create_bert_features(problem,
         # check whether tokenization changed the length
         if is_seq:
             target, tokenized_dict = seq_tag_label_handling(
-                tokenized_dict, target, tokenizer.pad_token)
+                tokenized_dict, target, '[PAD]')
 
             if len(target) != len(tokenized_dict['input_ids']):
                 raise ValueError(
@@ -149,6 +150,17 @@ def _create_bert_features(problem,
             }
         if problem_type in ['seq2seq_tag', 'seq2seq_text']:
             return_dict['%s_mask' % problem] = label_mask
+
+        if example_id < 10:
+            if isinstance(raw_inputs, dict):
+                for raw_input_name, raw_input in raw_inputs.items():
+                    LOGGER.info('{}: {}'.format(
+                        raw_input_name, str(raw_input)[:200]))
+            else:
+                LOGGER.info(str(raw_inputs)[:200])
+            for return_key, return_item in return_dict.items():
+                LOGGER.info('{}: {}'.format(
+                    return_key, str(return_item)[:200]))
         yield return_dict
 
 
@@ -280,10 +292,10 @@ def _create_multimodal_bert_features(problem,
     if problem_type == 'pretrain':
         raise NotImplementedError('Multimodal Pretraining is not implemented')
 
-    for example in example_list:
-        try:
+    for example_id, example in enumerate(example_list):
+        if mode != tf.estimator.ModeKeys.PREDICT:
             raw_inputs, raw_target = example
-        except ValueError:
+        else:
             raw_inputs = example
             raw_target = None
 
@@ -382,6 +394,17 @@ def _create_multimodal_bert_features(problem,
 
         if problem_type in ['seq2seq_tag', 'seq2seq_text']:
             return_dict['%s_mask' % problem] = label_mask
+
+        if example_id < 10:
+            if isinstance(raw_inputs, dict):
+                for raw_input_name, raw_input in raw_inputs.items():
+                    LOGGER.info('{}: {}'.format(
+                        raw_input_name, str(raw_input)[:200]))
+            else:
+                LOGGER.info(str(raw_inputs)[:200])
+            for return_key, return_item in return_dict.items():
+                LOGGER.info('{}: {}'.format(
+                    return_key, str(return_item)[:200]))
         yield return_dict
 
 
